from stomp.transport import Transport
from stomp.publisher import Publisher
from stomp.dispatcher import FrameDispatcher
from stomp.datastructures import Frame


# Legacy imports
from stomp import utils
from stomp import const
from stomp import datastructures
from stomp import exceptions


def get_version():
    return '0.2.0'


def transport_factory(dsn, enable_ssl=False, heartbeats=None,
    encoding='utf-8', defer=False, chunk_size=1024):
    """Create a new :class:`stomp.Transport` instance.

    Args:
        dsn (str): a Data Source Name (DSN) specifying the
            address, virtual host and login credentials of
            the STOMP server.
        enable_ssl (bool): indicates if the connection should
            be made using Secure Sockets Layer (SSL).
        heartbeats (tuple): a tuple of integers specifying the
            heartbeat policy. The first member is the minimum
            interval between client-server heartbeats; the
            second member is the minimum interval between
            server-client heartbeats.
        encoding (str): specifies the character encoding of the
            connection; default is UTF-8.
        defer (bool): defer the asynchronous receiving of 
            STOMP frames. If `defer` is ``True``, the caller
            must invoke :meth:`Transport.start()` to start
            receiving messages.
        chunk_size (int): number of bytes to receive per
            poll. 

    Returns:
        stomp.Transport

    Raises:
        ValueError: invalid DSN or heartbeat.
    """
    if heartbeats is not None:
        if tuple(map(type, heartbeats)) != (int, int):
            raise ValueError("`heartbeat` must be a tuple of integers.")
    username, password, host, port, vhost = utils.parse_dsn(dsn)
    
    transport = Transport(host, port, enable_ssl=enable_ssl,
        encoding=encoding, chunk_size=chunk_size)
    transport.connect()
    transport.login(vhost, username, password, heartbeats=heartbeats)
    if not defer:
        transport.start()
    return transport




