import operator
import functools

from stomp.const import EOL
from stomp.const import EOF
from stomp.frame.body import FrameBody
from stomp.frame.frametype import FrameType
from stomp.frame.header import FrameHeader


class Frame(object):
    __slots__ = ['_headers', '_frame_type', '_body', '_encoding']
    frame_type = property(lambda self: self._frame_type)
    headers = property(lambda self: {x.name:x.value for x in self._headers})

    @classmethod
    def fromraw(cls, raw_frame):
        return cls.parse(str(raw_frame))

    @classmethod
    def parse(cls, buf, encoding='utf-8'):
        """Initialize a new :class:`Frame` using a buffer holding a sequence
        of bytes received from the server.

        Args:
            buf (bytes): the data received from the server.
            encoding (str): specifies the encoding of the received message;
                default is 'utf-8'.

        Returns:
            Frame
        """
        data = buf.decode(encoding=encoding) if isinstance(buf, bytes)\
            else buf

        # The head and body of a stomp frame are separated by an empty
        # newline.
        head, body = data.split(EOL * 2, 1)

        # The first line of the head MUST contain the frame type.
        frame_type, *raw_headers = head.split(EOL)

        # The headers are separated by a newline.
        headers = [FrameHeader.parse(x) for x in raw_headers]

        return cls(frame_type, headers=headers, body=body[:body.index(EOF)],
            encoding=encoding)

    def __init__(self, frame_type, headers=None, body=None, encoding='utf-8'):
        """Initialize new :class:`Frame` instance.

        Args:
            frame_type (str): specifies the frame type.
            headers (list or dict): a list of :class:`stomp.frame.FrameHeader`
                instances or a dictionary mapping header names to values.
            body (str): the body of the message. May be None.
            encoding (str): specifies the encoding of the message.

        """
        self._frame_type = FrameType(frame_type) \
            if isinstance(frame_type, str) else frame_type

        # The headers may be passed as a list of FrameHeader instances or a
        # dictionary.
        headers = headers or []
        if isinstance(headers, dict):
            headers = [FrameHeader(k, v) for k, v in headers.items()]
        self._headers = headers
        self._body = FrameBody(body or '')
        self._encoding = encoding

    def add_header(self, name, value):
        """Adds a new header to the frame.

        Args:
            name (str): specifies the name of the header.
            value (str): specifies the value of the header.

        Returns:
            None
        """
        self._headers.append(FrameHeader(name, value))

    def send(self, func):
        """Send the frame to the remote server using `func`.

        Args:
            func: a callable that takes a byte-sequence as it's
                first argument.

        Returns:
            int
        """
        return func(bytes(self))

    def render(self):
        """Renders the message as a string object."""
        return str(self._frame_type) \
            + functools.reduce(operator.add, map(str, self._headers), '')\
            + EOL + str(self._body) + EOF

    def __bytes__(self):
        return bytes(self.render(), encoding=self._encoding)
